if (!Element.prototype.matches) {
    Element.prototype.matches = Element.prototype.msMatchesSelector ||
        Element.prototype.webkitMatchesSelector;
}

if (!Element.prototype.closest) {
    Element.prototype.closest = function (s) {
        var el = this;

        do {
            if (el.matches(s)) return el;
            el = el.parentElement || el.parentNode;
        } while (el !== null && el.nodeType === 1);
        return null;
    };
}

var Radzen = {
    getCookie : function (name) {
        var value = "; " + decodeURIComponent(document.cookie);
        var parts = value.split("; " + name + "=");
        if (parts.length == 2) return parts.pop().split(";").shift();
    },
    getCulture: function() {
        var cultureCookie = Radzen.getCookie(".AspNetCore.Culture");
        var uiCulture = cultureCookie ? cultureCookie.split('|').pop().split('=').pop() : null;
        return uiCulture || 'en-US';
    },
    numericOnPaste: function (e) {
        if (e.clipboardData) {
          var value = e.clipboardData.getData('text');

          if (value && /^-?\d*\.?\d*$/.test(value)) {
            return;
          }

          e.preventDefault();
        }
    },
    numericKeyPress: function (e) {
        if (e.metaKey || e.ctrlKey || e.keyCode == 9 || e.keyCode == 8 || e.keyCode == 13) {
            return;
        }

        var ch = String.fromCharCode(e.charCode);

        if (/^[-\d,.]$/.test(ch)) {
            return;
        }

        e.preventDefault();
    },
    openPopup: function (e, el, popup, isDropDown, additionalStyle) {
        const handler = function () {
            popup.isOpen = false;
            popup.style.display = 'none';
            if (popup.parentNode) {
                popup.parentNode.classList.remove('ui-dropdown-open');
            }

            window.removeEventListener('wheel', handler);
        };
        window.addEventListener('wheel', handler);

        var style = isDropDown ? `margin-left:-0.625rem; z-index: 1005; transform: translateY(3px); opacity: 1; display:block; position: fixed; ${Radzen.getPopupWidthAndTop(popup, isDropDown)};${additionalStyle}`
            : `margin-left: 0px; z-index: 1002; transform: translateY(0px); opacity: 1; position: fixed; ${Radzen.getPopupWidthAndTop(popup, isDropDown)};${additionalStyle}`;

        popup.isOpen = !popup.isOpen;

        popup.style.cssText = popup.isOpen ? style : 'display:none;';

        if (popup.parentNode) {
            if (popup.style.display == 'none') {
                popup.parentNode.classList.remove('ui-dropdown-open');
            } else {
                popup.parentNode.classList.add('ui-dropdown-open');
            }
        }
    },
    closePopup: function (e, el, popup) {
        if (!el.contains(e.relatedTarget) && popup) {
            popup.style.display = 'none';
            popup.isOpen = false;
            if (popup.parentNode) {
                popup.parentNode.classList.remove('ui-dropdown-open');
            }
        }
    },
    getInputValue: function (arg) {
        var input = (arg instanceof Element || arg instanceof HTMLDocument) ? arg : document.getElementById(arg);
        return input ? input.value : '';
    },
    setInputValue: function (arg,value) {
        var input = (arg instanceof Element || arg instanceof HTMLDocument) ? arg : document.getElementById(arg);
        if (input) {
            input.value = value;
        }
    },
    getPopupWidthAndTop: function (popup, isDropDown) {
        var el = popup.parentNode;
        var rect = el.getBoundingClientRect();
        var top = rect.top + rect.height;

        popup.style.cssText = 'visibility:hidden;'

        var windowHeight = window.innerHeight;
        var popupHeight = popup.offsetHeight;
        var offsetWithPopup = el.getBoundingClientRect().bottom + popupHeight;

        if (offsetWithPopup >= windowHeight) {
            top = top - rect.height - popupHeight;
        }

        popup.style.cssText = 'display:none;'

        return `width:calc(${rect.width + 'px'} - 1.425rem); top:${top}px;`;
    },
    readFileAsBase64: function (fileInput) {
        const readAsDataURL = function (fileInput) {
            return new Promise(function (resolve, reject) {
                const reader = new FileReader();
                reader.onerror = function () {
                    reader.abort();
                    reject(new Error("Error reading file."));
                };
                reader.addEventListener("load", function () {
                    resolve(reader.result);
                }, false);
                reader.readAsDataURL(fileInput.files[0]);
            });
        };

        return readAsDataURL(fileInput);
    },
    closeMenuItems: function (event) {
        var menu = event.target.closest('.menu');

        if (!menu) {
            var targets = document.querySelectorAll('.navigation-item-wrapper-active');

            if (targets) {
                for (var i = 0; i < targets.length; i++) {
                    Radzen.toggleMenuItem(targets[i], false);
                }
            }
            document.removeEventListener('click', Radzen.closeMenuItems);
        }
    },
    closeOtherMenuItems: function (current) {
        var targets = document.querySelectorAll('.navigation-item-wrapper-active');
        if (targets) {
            for (var i = 0; i < targets.length; i++) {
                var target = targets[i];
                var item = target.closest('.navigation-item');

                if (!item.contains(current)) {
                    Radzen.toggleMenuItem(target, false);
                }
            }
        }
    },
    toggleMenuItem: function (target, selected) {
        Radzen.closeOtherMenuItems(target);

        var item = target.closest(".navigation-item");

        if (selected === undefined) {
            selected = !item.classList.contains('navigation-item-active');
        }

        item.classList.toggle('navigation-item-active', selected);

        target.classList.toggle('navigation-item-wrapper-active', selected)

        var children = item.querySelector('.navigation-menu');

        if (children) {
            children.style.display = selected ? '' : 'none';
        }

        var icon = item.querySelector('.navigation-item-icon-children');

        if (icon) {
            var deg = selected ? '180deg' : 0;
            icon.style.transform = 'rotate(' + deg + ')';
        }

        document.removeEventListener('click', Radzen.closeMenuItems);
        document.addEventListener('click', Radzen.closeMenuItems);
    }
};